/***********************************************************************
Investigate how large floating-point values, Infinity, and NaN are
converted to integer types.
[01-Jan-2024]
***********************************************************************/

#include "config.h"
#include "headers.h"

#if defined(HAVE_LONG_LONG)
typedef long long int lli_t;
#define FMTLLD		"%lld"
#define FMTLLX		"0x%016llx"
#define LLI_CAST_NAME	"(long long int)"
#else
typedef long int lli_t;
#define FMTLLD		"%ld"
#define FMTLLX		"0x%08lx"
#define LLI_CAST_NAME	"(long int)"
#endif

#define PRINTF	(void)printf

int
main(void)
{
    volatile double x, y, z;
    lli_t i;

    PRINTF("Test of selected floating-point conversions to integers:\n");

    x = 0.0;
    y = 0.0;
    z = x / y;
    i = (lli_t)z;
    PRINTF("%sNaN        = %a = %.17g\n", "\n", z, z);
    PRINTF("%sNaN        = " FMTLLX " = \t" FMTLLD "\n", LLI_CAST_NAME, i, i);

    x = 1.0;
    y = 0.0;
    z = x / y;
    i = (lli_t)z;
    PRINTF("%sInf        = %a = %.17g\n", "\n", z, z);
    PRINTF("%sInf        = " FMTLLX " = \t" FMTLLD "\n", LLI_CAST_NAME, i, i);

    z = DBL_MAX;
    i = (lli_t)z;
    PRINTF("%sDBL_MAX    = %a = %.17g\n", "\n", z, z);
    PRINTF("%sDBL_MAX    = " FMTLLX " = \t" FMTLLD "\n", LLI_CAST_NAME, i, i);

    x = 0.0;
    y = 0.0;
    z = x / y;
    z = (copysign(1.0, z) > 0.0) ? copysign(z, -1.0) : copysign(z, +1.0);
    i = (lli_t)z;
    PRINTF("%s(-NaN)     = %a = %.17g\n", "\n", z, z);
    PRINTF("%s(-NaN)     = " FMTLLX " = \t" FMTLLD "\n", LLI_CAST_NAME, i, i);

    x = 1.0;
    y = 0.0;
    z = -x / y;
    i = (lli_t)z;
    PRINTF("%s(-Inf)     = %a = %.17g\n", "\n", z, z);
    PRINTF("%s(-Inf)     = " FMTLLX " = \t" FMTLLD "\n", LLI_CAST_NAME, i, i);

    z = -DBL_MAX;
    i = (lli_t)(z);
    PRINTF("%s(-DBL_MAX) = %a = %.17g\n", "\n", z, z);
    PRINTF("%s(-DBL_MAX) = " FMTLLX " = \t" FMTLLD "\n", LLI_CAST_NAME, i, i);

    z = 0.0;
    i = (lli_t)(z);
    PRINTF("%s(+0)       = %a = %.17g\n", "\n", z, z);
    PRINTF("%s(+0)       = " FMTLLX " = \t" FMTLLD "\n", LLI_CAST_NAME, i, i);

    z = copysign(0.0, -1.0);
    i = (lli_t)(z);
    PRINTF("%s(-0)       = %a = %.17g\n", "\n", z, z);
    PRINTF("%s(-0)       = " FMTLLX " = \t" FMTLLD "\n", LLI_CAST_NAME, i, i);

    return (EXIT_SUCCESS);
}
